<?php
// Copyright (c) 2005-2009, RedIRIS. All Rights Reserved.
//
// You may distribute under the terms of the GNU General Public License,
// as specified in the LICENSE file that was shipped with this distribution

include("Authz.php");

class AuthzDBA implements Authz {

    var $cfg;
    var $error;

    public function configure($file,$section) {
        $this->cfg['DB_Type'] = "db4";
        $this->cfg['Invite_Text_es'] = "Ha recibido una invitacion para acceder a ".$section.":\n\n#URL#";
        $this->cfg['Invite_Text_en'] = "You have received an invitation to access to ".$section.":\n\n#URL#";
        $this->cfg['Invite_Title_es'] = "Invitacion para unirse a ".$section;
        $this->cfg['Invite_Title_en'] = "Please join ".$section;

        $c = parse_ini_file($file,true);
        $params = Array (
            "DB_File",
            "Invite_DB_File",
            "Authz_Attribute"
        );
        
        foreach ($c['Authz_Main'] as $key => $value) {
            $this->cfg[$key] = $value;
        }
        foreach ($c[$section] as $key => $value) {
            $this->cfg[$key] = $value;
        }
        foreach ($params as $param) {
            if (!isset($this->cfg[$param])) {
                $this->error = "AuthzDBA::configure(): required parameter $param does not exist.";
            }
        }
    }

    public function getError() {
        return $this->error;
    }

    public function isAuthorized($user, $attrs) {
        $id = dba_open($this->cfg["DB_File"], "r", $this->cfg['DB_Type']);
        if (!$id) {
            $this->error = "AuthzDBA::isAuthorized(): cannot open database file ".$this->cfg['DB_File'].".";
            return false;
        } elseif (!(dba_exists($user, $id))) {
            $this->error = "AuthzDBA::isAuthorized(): key $user does not exist in database ".$this->cfg['DB_File'].".";
            dba_close($id);
            return false;
        }

        list($name, $attr, $mail) = explode("##", dba_fetch($user, $id));
        dba_close($id);

        $values = explode("|", $attrs[$name]);
        foreach ($values as $value) {
            if ($attr === $value) {
              return true;
            }
        }

        $this->error = "AuthzDbA::isAuthorized(): user ".$this->cfg['Authz_Attribute']." does not match. Expected '".
                        $value."', '".$attrs[$this->cfg['Authz_Attribute']]."' found.";
        return false;
    }

    public function getAuthorizedList() {
        $db = dba_open($this->cfg['DB_File'], "w", $this->cfg['DB_Type']);
        if (!$db) {
            $this->error = "AuthzDBA::getAuthorizedList(): cannot open database file ".$this->cfg['DB_File'].".";
            return false;
        }

        $list = array();
        $key = dba_firstkey($db);
        while ($key) {
            list($name, $attr, $id) = explode("##", dba_fetch($key, $db));
            $list[$key]['attribute'] = $attr;
            $list[$key]['attribute_name'] = $name;
            $list[$key]['mail'] = $id;

            $key = dba_nextkey($db);
        }

        dba_close($db);
        return $list;
    }

    public function getPendingInvites() {
        $db = dba_open($this->cfg['Invite_DB_File'], "w", $this->cfg['DB_Type']);
        if (!$db) {
            $this->error = "AuthzDBA::getPendingInvites(): cannot open database file ".$this->cfg['Invite_DB_File'].".";
            return false;
        }

        $list = array();
        $key = dba_firstkey($db);
        while ($key) {
            $mail = dba_fetch($key, $db);
            $list[$key] = $mail;

            $key = dba_nextkey($db);
        }

        dba_close($db);
        return $list;
    }

    public function authorize($user,$attrs,$ref,$force = false) {
        $idb = dba_open($this->cfg['Invite_DB_File'], "w", $this->cfg['DB_Type']);
        if (!$idb && !$force) {
            $this->error = "AuthzDBA::authorize(): cannot open database file ".$this->cfg['Invite_DB_File'].".";
            return false;
        }

        $adb = dba_open($this->cfg["DB_File"], "c", $this->cfg['DB_Type']);
        if (!$adb) {
            $this->error = "AuthzDBA::authorize(): cannot open database file ".$this->cfg['DB_File'].".";
            return false;
        }

        $attr_names = $this->cfg['Authz_Attribute'];
        if (!is_array($attr_names)) {
            $a[] = $attr_names;
            $attr_names = $a;
        }

        if (!$force && !(dba_exists($ref, $idb))) {
            if (dba_exists($user, $adb)) {
                list($name, $attr, $mail) = explode("##", dba_fetch($user, $adb));
                // DESCOMENTAR!!!!!!!
                //if (isset($attrs[$name]) && $attr === $attrs[$name]) {
                    dba_close($idb);
                    dba_close($adb);
                    return true;
                //}
            } 
            $this->error = "AuthzDBA::authorize(): no such invite $ref registered.";
            dba_close($idb);
            return false;
        }

        $mail = dba_fetch($ref, $idb);

        $name = "";
        $value = "";
        foreach ($attr_names as $attr_name) {
            if (!empty($attrs[$attr_name])) {
                $name = $attr_name;
                $values = explode("|", $attrs[$name]);
                $value = $values[0];
                break;
            }
        }

        if (empty($name)) {
            $this->error = "AuthzDBA::authorize(): could not find any attribute in user assertion";
            return false;
        }

        dba_replace($user, implode("##",array($name, $value, $mail)), $adb);
        dba_delete($ref,$idb);
        dba_close($idb);
        dba_close($adb);
        return true;
    }

    public function revoke($mail) {
        $mail = strtolower($mail);
        $db = dba_open($this->cfg['DB_File'], "w", $this->cfg['DB_Type']);
        if (!$db) {
            $this->error = "AuthzDBA::revoke(): cannot open database file ".$this->cfg['DB_File'].".";
            return false;
        }

        $key = dba_firstkey($db);
        while ($key) {
            list($name, $attr, $id) = explode("##", dba_fetch($key, $db));
            if ($mail === $id) {
                dba_delete($key, $db);
                dba_close($db);
                return true;
            }
            $key = dba_nextkey($db);
        }

        dba_close($db);
        $this->error = "AuthzDBA::revoke(): cannot find user ".$mail;
        return false;
    }

    public function invite($mail,$lang,$url) {
        $ref = mt_rand();
        $id = dba_open($this->cfg['Invite_DB_File'], "c", $this->cfg['DB_Type']);
        if (!$id) {
            $this->error = "AuthzDBA::invite(): cannot open database file ".$this->cfg['Invite_DB_File'].".";
            return false;
        }
        dba_replace($ref, strtolower($mail), $id);
        dba_close($id);
        $sep = (!strstr($url,"?")) ? "?" : "&";
        $text = $this->cfg['Invite_Text_'.$lang];
        $text = preg_replace("/#URL#/",$url.$sep."ref=".$ref,$text);
        $subject = $this->cfg['Invite_Title_'.$lang];
        $headers = (isset($this->cfg['Admin_Email'])) ? "From: ".$this->cfg['Admin_Email'] : "";
        $additional_params = (isset($this->cfg['Admin_Email'])) ? "-f ".$this->cfg['Admin_Email'] : "";
        mail($mail,$subject,$text,$headers, $additional_params);
        return true;
    }

    public function removeInvite($ref) {
        $db = dba_open($this->cfg['Invite_DB_File'], "c", $this->cfg['DB_Type']);
        if (!$db) {
            $this->error = "AuthzDBA::removeInvite(): cannot open database file ".$this->cfg['Invite_DB_File'].".";
            return false;
        }
        $rslt = dba_delete($ref, $db);
        dba_close($db);
        return $rslt;
    }
}

?>
